#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the OpenPost project
"""

import os
import sys
import time
import unittest
from contextlib import contextmanager

import cli.openpost as test_module


@contextmanager
def suppress_stdout():
    with open(os.devnull, "w") as devnull:
        old_stdout = sys.stdout
        sys.stdout = devnull
        try:
            yield
        finally:
            sys.stdout = old_stdout


@contextmanager
def suppress_allout():
    with open(os.devnull, "w") as devnull:
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = devnull
        sys.stderr = devnull
        try:
            yield
        finally:
            sys.stdout = old_stdout
            sys.stderr = old_stderr


class ArgsObject():
    """
    Temporary object used for testing to mimic the args object
    returned by argparse.
    """
    date_name = False
    random_name = False
    FILENAME = None
    FILEPATH = None
    keep_file = False
    SECONDS = None
    url = None
    post_data = None


class MyTests(unittest.TestCase):

    def test_name_date(self):
        name = test_module.date_filename()
        self.assertEqual(len(name), 19)
        self.assertTrue(name.startswith(time.strftime('%Y%m%d')))
        self.assertTrue(name.endswith('.html'))

    def test_name_random(self):
        name = test_module.random_filename()
        self.assertEqual(len(name), 37)
        self.assertTrue(name.endswith('.html'))

    def test_name_01(self):
        args = ArgsObject()
        args.date_name = True
        args.random_name = False
        args.FILENAME = 'Test File Name'
        name = test_module.make_file_name(args)
        self.assertEqual(len(name), 19)
        self.assertTrue(name.startswith(time.strftime('%Y%m%d')))
        self.assertTrue(name.endswith('.html'))

    def test_name_02(self):
        args = ArgsObject()
        args.date_name = False
        args.random_name = True
        args.FILENAME = 'Test File Name'
        name = test_module.make_file_name(args)
        self.assertEqual(len(name), 37)
        self.assertTrue(name.endswith('.html'))

    def test_name_03(self):
        args = ArgsObject()
        args.date_name = False
        args.random_name = False
        args.FILENAME = 'Test File Name'
        name = test_module.make_file_name(args)
        self.assertEqual(len(name), 19)
        self.assertTrue(name.endswith('.html'))
        self.assertTrue(name.startswith('Test File Name'))

    def test_name_04(self):
        args = ArgsObject()
        args.date_name = False
        args.random_name = False
        name = test_module.make_file_name(args)
        self.assertEqual(name, test_module.DEFAULT_HTML_FILE)
        self.assertTrue(name.endswith('.html'))

    def test_make_file_path_01(self):
        args = ArgsObject()
        name = test_module.make_file_path(args)
        self.assertEqual(name, '.')

    def test_make_file_path_02(self):
        args = ArgsObject()
        args.FILEPATH = 'test_FILE_path'
        name = test_module.make_file_path(args)
        self.assertEqual(name, 'test_FILE_path')

    def test_make_file_path_03(self):
        args = ArgsObject()
        args.FILEPATH = 'test'
        name = test_module.make_file_path(args)
        self.assertEqual(name, 'test')

    def test_post_data_single(self):
        data = test_module.make_form_data_string(['abc=123'])
        self.assertTrue('<input type="hidden" name="abc" value="123">' in data)

    def test_post_data_multiple(self):
        data = test_module.make_form_data_string(['abc=123', 'xyz=456'])
        self.assertTrue('<input type="hidden" name="abc" value="123">' in data)
        self.assertTrue('<input type="hidden" name="xyz" value="456">' in data)

    def test_post_data_case(self):
        data = test_module.make_form_data_string(['abc=123'])
        self.assertTrue('<input type="hidden" name="ABC" value="123">' not in data)

    def test_test_url_01(self):
        data = test_module.test_url(1)
        self.assertEqual(data, 103)

    def test_test_url_02(self):
        data = test_module.test_url('')
        self.assertEqual(data, 101)

    def test_test_url_03(self):
        data = test_module.test_url(' ')
        self.assertEqual(data, 101)

    def test_test_url_04(self):
        data = test_module.test_url('embedded space')
        self.assertEqual(data, 102)

    def test_test_url_05(self):
        data = test_module.test_url('localhost')
        self.assertEqual(data, 0)

    def test_test_url_06(self):
        data = test_module.test_url('192.168.1.1')
        self.assertEqual(data, 0)

    def test_test_url_07(self):
        data = test_module.test_url('www.somewhere.com')
        self.assertEqual(data, 0)

    def test_test_url_08(self):
        data = test_module.test_url('http://www.somewhere.com')
        self.assertEqual(data, 0)

    def test_test_url_09(self):
        data = test_module.test_url('https://www.somewhere.com')
        self.assertEqual(data, 0)

    def test_test_url_10(self):
        data = test_module.test_url('www.somewhere.com/')
        self.assertEqual(data, 0)

    def test_test_url_11(self):
        data = test_module.test_url('www.somewhere.com/subdir')
        self.assertEqual(data, 0)

    def test_test_url_12(self):
        data = test_module.test_url('www.somewhere.com/subdir/')
        self.assertEqual(data, 0)

    def test_test_ttl_01(self):
        args = ArgsObject()
        args.SECONDS = None
        data = test_module.make_time_to_live(args)
        self.assertEqual(data, test_module.DEFAULT_TIME_TO_LIVE)

    def test_test_ttl_02(self):
        args = ArgsObject()
        args.SECONDS = 10
        data = test_module.make_time_to_live(args)
        self.assertEqual(data, 10)

    def test_test_ttl_03(self):
        args = ArgsObject()
        args.SECONDS = 13.5
        data = test_module.make_time_to_live(args)
        self.assertEqual(data, 13.5)

    def test_exit_missing(self):
        args = ArgsObject()
        args.url = ''
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.exit_with_error()
        self.assertEqual(err.exception.code, -1)
        # unittest.main(exit=False)

    def test_exit_unknown(self):
        args = ArgsObject()
        args.url = ''
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.exit_with_error(-1)
        self.assertEqual(err.exception.code, -1)
        # unittest.main(exit=False)

    def test_exit_101(self):
        args = ArgsObject()
        args.url = ''
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.exit_with_error(101)
        self.assertEqual(err.exception.code, 101)
        # unittest.main(exit=False)

    def test_ttl_104_1(self):
        args = ArgsObject()
        args.SECONDS = -1
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_time_to_live(args)
        self.assertEqual(err.exception.code, 104)
        # unittest.main(exit=False)

    def test_ttl_104_2(self):
        args = ArgsObject()
        args.SECONDS = 61
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_time_to_live(args)
        self.assertEqual(err.exception.code, 104)
        # unittest.main(exit=False)

    def test_path_105_1(self):
        args = ArgsObject()
        args.FILEPATH = ''
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_file_path(args)
        self.assertEqual(err.exception.code, 105)
        # unittest.main(exit=False)

    def test_path_105_2(self):
        args = ArgsObject()
        args.FILEPATH = ' '
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_file_path(args)
        self.assertEqual(err.exception.code, 105)
        # unittest.main(exit=False)

    def test_path_106(self):
        args = ArgsObject()
        args.FILEPATH = 'cli/openpost.py'
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_file_path(args)
        self.assertEqual(err.exception.code, 106)
        # unittest.main(exit=False)

    def test_name_107_1(self):
        args = ArgsObject()
        args.FILENAME = ''
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_file_name(args)
        self.assertEqual(err.exception.code, 107)
        # unittest.main(exit=False)

    def test_name_107_2(self):
        args = ArgsObject()
        args.FILENAME = ' '
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_file_name(args)
        self.assertEqual(err.exception.code, 107)
        # unittest.main(exit=False)

    def test_data_108_1(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_form_data_string('')
        self.assertEqual(err.exception.code, 108)
        # unittest.main(exit=False)

    def test_data_108_2(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_form_data_string(0)
        self.assertEqual(err.exception.code, 108)
        # unittest.main(exit=False)

    def test_data_108_3(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_form_data_string(1.5)
        self.assertEqual(err.exception.code, 108)
        # unittest.main(exit=False)

    def test_data_108_4(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_form_data_string(('a', 'b'))
        self.assertEqual(err.exception.code, 108)
        # unittest.main(exit=False)

    def test_data_108_5(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_form_data_string({'key': 'value'})
        self.assertEqual(err.exception.code, 108)
        # unittest.main(exit=False)

    def test_data_109(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_form_data_string(['no equals sign'])
        self.assertEqual(err.exception.code, 109)
        # unittest.main(exit=False)

    def test_data_110(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_form_data_string(['=no key'])
        self.assertEqual(err.exception.code, 110)
        # unittest.main(exit=False)

    def test_data_111(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.make_form_data_string([])
        self.assertEqual(err.exception.code, 111)
        # unittest.main(exit=False)

    def test_main_0_1(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.parse_command_arguments(['--help'])
        self.assertEqual(err.exception.code, 0)
        # unittest.main(exit=False)

    def test_main_0_2(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_stdout():
                test_module.parse_command_arguments(['-h'])
        self.assertEqual(err.exception.code, 0)
        # unittest.main(exit=False)

    def test_main_2_1(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_allout():
                test_module.parse_command_arguments([])
        self.assertEqual(err.exception.code, 2)
        # unittest.main(exit=False)

    # Test disabled because missing key=value pairs no longer caught in parse_command_arguments module.

    # def test_main_2_2(self):
    #     with self.assertRaises(SystemExit) as err:
    #         with suppress_allout():
    #             test_module.parse_command_arguments(['localhost'])
    #     self.assertEqual(err.exception.code, 2)
    #     # unittest.main(exit=False)

    def test_main_2_3(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_allout():
                test_module.parse_command_arguments(['localhost', 'key=value', '-r', '-d'])
        self.assertEqual(err.exception.code, 2)
        # unittest.main(exit=False)

    def test_main_2_4(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_allout():
                test_module.parse_command_arguments(['localhost', 'key=value', '-r', '-f', 'filename'])
        self.assertEqual(err.exception.code, 2)
        # unittest.main(exit=False)

    def test_main_2_5(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_allout():
                test_module.parse_command_arguments(['localhost', 'key=value', '-d', '-f', 'filename'])
        self.assertEqual(err.exception.code, 2)
        # unittest.main(exit=False)

    def test_main_2_6(self):
        with self.assertRaises(SystemExit) as err:
            with suppress_allout():
                test_module.parse_command_arguments(['localhost', 'key=value', '-k', '-t', '5'])
        self.assertEqual(err.exception.code, 2)
        # unittest.main(exit=False)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the OpenPost project
"""

import os
import sys
import unittest
from contextlib import contextmanager

import openpost as test_module


@contextmanager
def suppress_stdout():
    with open(os.devnull, "w") as devnull:
        old_stdout = sys.stdout
        sys.stdout = devnull
        try:
            yield
        finally:
            sys.stdout = old_stdout


@contextmanager
def suppress_allout():
    with open(os.devnull, "w") as devnull:
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = devnull
        sys.stderr = devnull
        try:
            yield
        finally:
            sys.stdout = old_stdout
            sys.stderr = old_stderr


class MyTests(unittest.TestCase):

    def test_initialize_01(self):
        poster = test_module.OpenPost()
        self.assertEqual(poster.url, None)
        self.assertEqual(poster.file_name, 'OpenPost.html')
        self.assertFalse(poster.keep_file)
        self.assertEqual(poster.time_to_live, 5)
        self.assertEqual(poster.form_data, {})
        self.assertEqual(poster.version(), test_module.__version__)

    def test_initialize_02(self):
        poster = test_module.OpenPost('test_url', 'test_file_name', True, 10.5, {'one': 'This is the number one.'})
        self.assertEqual(poster.url, 'test_url')
        self.assertEqual(poster.file_name, 'test_file_name')
        self.assertTrue(poster.keep_file)
        self.assertEqual(poster.time_to_live, 10.5)
        self.assertEqual(len(poster.form_data.keys()), 1)
        self.assertEqual(list(poster.form_data.keys()), ['one'])

    def test_edit_properties(self):
        poster = test_module.OpenPost()
        poster.url = 'test_url'
        self.assertEqual(poster.url, 'test_url')
        poster.file_name = 'test_file_name'
        self.assertEqual(poster.file_name, 'test_file_name')
        poster.keep_file = True
        self.assertTrue(poster.keep_file)
        poster.time_to_live = 10.5
        self.assertEqual(poster.time_to_live, 10.5)
        poster.form_data = {'one': 'This is the number one.'}
        self.assertEqual(len(poster.form_data.keys()), 1)
        self.assertEqual(list(poster.form_data.keys()), ['one'])

    def test_keys(self):
        poster = test_module.OpenPost()
        poster.add_key('one', 'This is the number one.')
        self.assertEqual(len(poster.form_data.keys()), 1)
        self.assertEqual(list(poster.form_data.keys()), ['one'])
        poster.clear_data()
        self.assertEqual(poster.form_data, {})
        poster.add_key('one', 'This is the number one.')
        self.assertEqual(len(poster.form_data.keys()), 1)
        self.assertEqual(list(poster.form_data.keys()), ['one'])
        poster.add_key('two', 'This is the number two.')
        self.assertEqual(len(poster.form_data.keys()), 2)
        poster.delete_key('one')
        self.assertEqual(len(poster.form_data.keys()), 1)
        self.assertEqual(list(poster.form_data.keys()), ['two'])

    def test_validate_ttl(self):
        poster = test_module.OpenPost()
        self.assertEqual(poster._validate_ttl(0), 0)
        self.assertEqual(poster._validate_ttl('15'), 15)
        self.assertEqual(poster._validate_ttl(15.000), 15)
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_ttl('')
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_ttl('zero')
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_ttl(-1)
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_ttl(61)

    def test_validate_data(self):
        poster = test_module.OpenPost()
        self.assertEqual(poster._validate_data({}), {})
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_data('string')
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_data(None)
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_data(['list'])
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_data(1)
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_data(1.5)

    def test_validate_url(self):
        poster = test_module.OpenPost()
        self.assertEqual(poster._validate_url('localhost'), 'localhost')
        self.assertEqual(poster._validate_url('https://some.test.url'), 'https://some.test.url')
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_url(None)
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_url('')
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_url(' ')
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_url('contains space')
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_url({'one': 1})
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_url(['one'])
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_url(1)
        with self.assertRaises(ValueError):
            with suppress_allout():
                poster._validate_url(1.5)

    def test_make_filename(self):
        poster = test_module.OpenPost()
        self.assertEqual(poster._make_filename(None), 'OpenPost.html')
        self.assertEqual(poster._make_filename(''), 'OpenPost.html')
        self.assertEqual(poster._make_filename(' '), 'OpenPost.html')
        self.assertEqual(poster._make_filename(-1), '-1.html')
        self.assertEqual(poster._make_filename(10.50), '10.5.html')
        self.assertEqual(poster._make_filename('fred.html'), 'fred.html')
        self.assertEqual(poster._make_filename(' fred.html'), 'fred.html')
        self.assertEqual(poster._make_filename('fred.html '), 'fred.html')
        self.assertEqual(poster._make_filename('fred.htm'), 'fred.htm.html')



#! /usr/bin/env python
#
# Copyright (C) 2020 Bob Swift
# <bswift@rsds.ca>

import os
from setuptools import setup

DESCRIPTION = """A module to help prepare and execute html POST requests."""
DIST_NAME = "openpost"
# PKG_NAME = DIST_NAME + '-rdswift'   # use only for testing on https://test.pypi.org
PKG_NAME = DIST_NAME
AUTH_NAME = "Bob Swift"
AUTH_EMAIL = 'bswift@rsds.ca'
MAINT_NAME = AUTH_NAME
MAINT_EMAIL = AUTH_EMAIL
URL = "https://github.com/rdswift/OpenPost"
LICENSE = "GPLv3"
DOWNLOAD_URL = "https://github.com/rdswift/OpenPost"

with open("./README.md", "r") as f:
    LONG_DESCRIPTION = f.read()

with open("./openpost/__init__.py", "r") as f:
    lines = f.readlines()
    for line in lines:
        if line.startswith("__version__"):
            __version__ = line.split("= ")[-1].strip().strip('"')
            break

if __name__ == "__main__":
    if os.path.exists("MANIFEST.in"):
        os.remove("MANIFEST.in")

    setup(
        name=PKG_NAME,
        author=AUTH_NAME,
        author_email=AUTH_EMAIL,
        # maintainer=MAINT_NAME,
        # maintainer_email=MAINT_EMAIL,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        long_description_content_type="text/markdown",
        license=LICENSE,
        url=URL,
        download_url=DOWNLOAD_URL,
        packages=["openpost"],
        version=__version__,
        zip_safe=False,  # the package can run out of an .egg file
        include_package_data=False,
        classifiers=[
            "Intended Audience :: Developers",
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 3",
            "Topic :: Internet :: WWW/HTTP",
        ],
        # python_requires='>=3.5',
        platforms="any",
        # package_data={},
        # scripts=[],
        # install_requires=[],
        # extras_require={},
    )


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#################################################################################
#                                                                               #
#   OpenPost - Opens a POST request from the command line in a browser window   #
#   Copyright (C) 2019 Bob Swift (rdswift)                                      #
#                                                                               #
#   This program is free software: you can redistribute it and/or modify it     #
#   under the terms of the GNU General Public License as published by the       #
#   Free Software Foundation, either version 3 of the License, or (at your      #
#   option) any later version.                                                  #
#                                                                               #
#   This program is distributed in the hope that it will be useful, but         #
#   WITHOUT ANY WARRANTY; without even the implied warranty of                  #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           #
#   General Public License for more details.                                    #
#                                                                               #
#   You should have received a copy of the GNU General Public License along     #
#   with this program.  If not, see <https://www.gnu.org/licenses/>.            #
#                                                                               #
#################################################################################

import os
# import html
# import re
import time
import webbrowser

__version__ = "0.1"


class OpenPost():

    HTML_TEMPLATE = """\
<html>
  <head>
    <title>OpenPost Redirector</title>
  </head>
  <body onLoad="javascript: document.getElementById('postform').submit();">
    <p>Loading...</p>
    <form method="post" name="postform" id="postform" action="{0}">
{1}
    </form>
  </body>
</html>
"""

    def __init__(self, url=None, file_name='OpenPost.html', keep_file=False, time_to_live=5, form_data={}):
        """Creates an html POST request file and allows opening in a browser window.

        Keyword Arguments:
            url {str} -- The url for the action in the POST (default: None)
            file_name {str} -- Path and name of the output html file (default: 'OpenPost.html')
            keep_file {bool} -- Keep the output html file after opening in browser (default: False)
            time_to_live {float} -- Number of seconds to delay before removing the output html file (0-60) (default: 5)
            form_data {dict} -- The key:value data to include in the POST request (default: {})
        """
        self.form_data = self._validate_data(form_data)
        self.url = url
        self.keep_file = keep_file
        self.file_name = file_name
        self.time_to_live = time_to_live
        self.written = False

    @staticmethod
    def version():
        """Returns the version number of the module.

        Returns:
            {str} -- The version number
        """
        return __version__

    @staticmethod
    def _validate_ttl(ttl):
        """Validates the time-to-live value for the output html file after sending the POST request.

        Arguments:
            ttl {float} -- Number of seconds to delay before removing the output html file

        Raises:
            ValueError: Time to live out of range (0-60)

        Returns:
            {float} -- Valid time-to-live value
        """
        temp = float(ttl)
        if 0 <= temp <= 60:
            return temp
        else:
            raise ValueError('Time to live out of range (0-60)')

    @staticmethod
    def _validate_data(form_data):
        """Validate the data to be used in the form

        Arguments:
            form_data {dict} -- Dictionary of key:value data to include in the POST request

        Raises:
            ValueError: Form_data not a dictionary

        Returns:
            {dict} -- The key:value data to include in the POST request
        """
        if isinstance(form_data, dict):
            return form_data
        else:
            raise ValueError('Form_data not a dictionary')

    @staticmethod
    def _validate_url(url):
        """Very basic validation of the submitted url.

        Arguments:
            url {str} -- URL to validate

        Raises:
            ValueError: Invalid url.

        Returns:
            {str} -- Validated url
        """
        if url and isinstance(url, str):
            temp = url.strip()
            # r'^(ftp|https?)://[^\s"]+$'
            if temp and ' ' not in temp:
                return temp
        raise ValueError('Invalid url.')

    @staticmethod
    def _make_filename(name):
        """Validate the output file name for the POST html file.

        Arguments:
            name {str} -- Path and file name

        Returns:
            {str} -- Validated file path and name
        """
        temp = str(name).strip()
        if name and temp:
            if not temp.endswith('.html'):
                temp += '.html'
            return temp
        else:
            return 'OpenPost.html'

    def clear_data(self):
        """Clears the data used for the POST request form.
        """
        self.form_data = {}
        self.written = False

    def add_key(self, key, value):
        """Add or update a data key used for the POST request form.

        Arguments:
            key {str} -- Key used in the form
            value {str} -- Value for the specified key
        """
        self.form_data[key] = str(value)
        self.written = False

    def delete_key(self, key):
        """Remove a data key used for the POST request form.

        Arguments:
            key {str} -- Key to be removed from the form
        """
        self.form_data.pop(key, None)
        self.written = False

    def make_html(self):
        """Make the content of the output html file.

        Returns:
            {str} -- The content of the html file, or '' if an error
        """
        url = self._validate_url(self.url)
        data = self._validate_data(self.form_data)
        if not data:
            return ''
        form_content = ''
        for key in data.keys():
            # html_key = html.escape(key)
            # html_value = html.escape(str(data[key]).strip())
            # form_content += "<textarea name='{0}' id='{0}' form='postform' style='display: none;'>{1}</textarea>\n".format(html_key, html_value)
            form_content += "<textarea name='{0}' id='{0}' form='postform' style='display: none;'>{1}</textarea>\n".format(key, str(data[key]).strip())
        return self.HTML_TEMPLATE.format(url, form_content)

    def write_html(self):
        """Prepare and write the output html file.

        Returns:
            {bool} -- True if the file was successfully written, otherwise false
        """
        self.written = False
        filename = self._make_filename(self.file_name)
        html = self.make_html()
        if not html:
            return False
        with open(filename, 'w', encoding='utf-8') as output_file:
            output_file.write(html)
        self.written = True
        return True

    def send_post(self):
        """Open the output POST html file in the default web browser, automatically writing the
        output html file if it has not already been written.  Automatically removes the output
        file after the specified time delay unless the keep_file flag has been set.

        Returns:
            {bool} -- True if the file was successfully opened, otherwise false
        """
        if not self.written:
            if not self.write_html():
                return False
        filename = self._make_filename(self.file_name)
        webbrowser.open_new_tab(filename)

        #   Remove temporary HTML file
        if not self.keep_file:
            time.sleep(self.time_to_live)
            if os.path.exists(filename):
                os.remove(filename)

        return True
